# import unittest
# from unittest.mock import patch
# from datetime import datetime

# import sys
# import os

# sys.path.insert(0, os.path.abspath(os.path.dirname(__file__)))

# from jgtutils.jgtwslhelper import _mkbash_cmd_string_jgtfxcli_range,resolve_cli_path


# class TestMkBashCmdStringJgtfxcliRange(unittest.TestCase):
#     @patch("jgtwslhelper.resolve_cli_path")
#     @patch("jgtwslhelper.jgtcommon.tlid_range_to_jgtfxcon_start_end_str")
#     def test_mkbash_cmd_string_jgtfxcli_range(
#         self, mock_tlid_range_to_jgtfxcon_start_end_str, mock_resolve_cli_path
#     ):
#         # Arrange
#         instrument = "EUR/USD"
#         timeframe = "H1"
#         tlid_range = "2101010000_2101012359"
#         cli_path = resolve_cli_path()
#         verbose_level = 2
#         date_from = "20220101"
#         date_to = "20220101"

#         #mock_resolve_cli_path.return_value = cli_path
#         mock_tlid_range_to_jgtfxcon_start_end_str.return_value = (date_from, date_to)

#         expected_bash_command = f'pwd;{cli_path} -i "{instrument}" -t "{timeframe}" -s {date_from} -e {date_to} -v {verbose_level}'

#         # Act
#         result = _mkbash_cmd_string_jgtfxcli_range(
#             instrument, timeframe, tlid_range, cli_path, verbose_level
#         )

#         # Assert
#         mock_resolve_cli_path.assert_called_once_with(cli_path)
#         mock_tlid_range_to_jgtfxcon_start_end_str.assert_called_once_with(tlid_range)
#         self.assertEqual(result, expected_bash_command)


# if __name__ == "__main__":
#     unittest.main()




# %% Imports
import sys
import os

sys.path.insert(0, os.path.abspath(os.path.dirname(__file__)))

from dotenv import load_dotenv

# Load environment variables from .env file
load_dotenv()

import plotly.graph_objects as go
from plotly.subplots import make_subplots
import panel as pn

import matplotlib.pyplot as plt

plt.rcParams["figure.max_open_warning"] = 100

import pandas as pd
from jgtpy import JGTADS as ads, adshelper as ah, JGTPDSP as pds, JGTCDS as cds
from jgtpy import JGTChartConfig

import tlid

#%% Creating a default chart configuration
def create_default_chart_config():
    cc = JGTChartConfig.JGTChartConfig()
    cc.saucer_marker_size = 14
    cc.ac_signals_marker_size = 14
    cc.fig_ratio_x = 31
    cc.fig_ratio_y = 16
    cc.nb_bar_on_chart = 300
    cc.plot_style = "yahoo"
    return cc
  
#%% Pto Panel 240209


#support crop_last_dt="2022-10-13 13:45:00"
def generate_market_snapshots(instruments:str, timeframes:str, html_outdir_root:str=None,cc:JGTChartConfig=None,crop_last_dt:str=None, show_chart=False, show_tabs=False,width=2550, height=1150,save_fig_image=True,save_cds_data=True):
  if cc is None:
    cc = create_default_chart_config()
  if html_outdir_root is None:
    # Read the environment variable
    jgtpy_data_dir = os.environ.get("JGTPY_DATA")

    # Check if the environment variable is set
    if jgtpy_data_dir is not None:
      # Join the environment variable with "charts" to create a new directory path
      scn_chart_dir = os.path.join(jgtpy_data_dir, "charts")
      
      # Create the directory if it doesn't exist
      os.makedirs(scn_chart_dir, exist_ok=True)
      
      # Set html_outdir_root to the new directory
      html_outdir_root = scn_chart_dir
    else:
      print("Environment variable JGTPY_DATA is not set. Default to ./charts.")
      html_outdir_root = "./charts"
  timeframes = timeframes.split(",")
  perspectives = {}
  ptabs = pn.Tabs(width=width, height=height)

  for i in instruments.split(","):
    try:
      print(f"-------------{i}-------------------")

      figures = {}
      success = False
      for t in timeframes:
        print(i, t)
        f, ax, _ = ads.plot(i, t, show=show_chart, cc=cc, crop_last_dt=crop_last_dt,plot_ao_peaks=True)
        f.title = t
        figures[t] = f
        fnout, fnoutcsv = _mk_fnoutputs(html_outdir_root, i, t)
        
        if save_fig_image:        
          f.savefig(fnout)
        if save_cds_data:
          _.to_csv(fnoutcsv)

      tabs = pn.Tabs(width=width, height=height)

      for t in timeframes:
        tabs.append((t, figures[t]))

      if show_tabs:
        tabs.show()
        
      
      if crop_last_dt is not None:
        #cldt_fnstr=crop_last_dt.replace("/","-").replace(" ","_").replace(":","") 
        cldt_fnstr=tlid.strdt(crop_last_dt)
        tabs.title = i + " - " + crop_last_dt
        html_fname = i.replace("/", "-")+"_"+ cldt_fnstr + ".html"
      else:
        cldt_fnstr =tlid.get_minutes()
        tabs.title = i
        html_fname = i.replace("/", "-") + ".html"
      print(html_fname)
      
      html_output_filepath = f"{html_outdir_root}/pto-mksg-" + html_fname

      tabs.save(html_output_filepath, embed=True)

      perspectives[i] = tabs

      ptabs.append((i, tabs))
    except:
      print("An error occurred while processing:", i)
      pass

  full_html_output_filepath = f"{html_outdir_root}/pto-all-mksg.html"
  print(full_html_output_filepath)

  ptabs.save(full_html_output_filepath, embed=True)
  print("Saved:", full_html_output_filepath)

def _mk_fnoutputs(html_outdir_root, i, t):
    povfn = i.replace("/", "-") + "_" + t
    fnout = html_outdir_root + "/" + povfn + ".png"
    fnoutcsv = html_outdir_root + "/" + povfn + ".cds.csv"
    return fnout,fnoutcsv


#!/usr/bin/env python

import sys
import os
sys.path.insert(0, os.path.abspath(os.path.dirname(__file__)))

# import .

from jgtutils import jgtconstants as constants,jgtcommon as jgtcommon,jgtwslhelper as wsl
import argparse
import JGTPDSP as pds
import JGTCDS as cds
import JGTADS as ads
from JGTChartConfig import JGTChartConfig


import pandas as pd

def parse_args():
    parser = argparse.ArgumentParser(description='Process command parameters.')
    #jgtfxcommon.add_main_arguments(parser)
    jgtcommon.add_instrument_timeframe_arguments(parser)
    jgtcommon.add_date_arguments(parser)
    jgtcommon.add_tlid_range_argument(parser)
    jgtcommon.add_max_bars_arguments(parser)
    #jgtcommon.add_output_argument(parser)
    #jgtfxcommon.add_quiet_argument(parser)
    jgtcommon.add_verbose_argument(parser)
    jgtcommon.add_ads_argument(parser)
    #jgtcommon.add_cds_argument(parser)
    args = parser.parse_args()
    return args


def main():
    cc=JGTChartConfig()
    args = parse_args()
    instrument = args.instrument
    timeframe = args.timeframe
    quotes_count = args.quotescount
    cc.nb_bar_on_chart=quotes_count
    
    date_from = None
    date_to = None
    tlid_range = None
    if args.tlidrange:
        #@STCGoal Get range prices from cache or request new
        tlid_range = args.tlidrange
        print("FUTURE Support for tlid range")
        tmpcmd = wsl._mkbash_cmd_string_jgtfxcli_range(instrument,timeframe,tlid_range)
        print("> " + tmpcmd)
        print("-----------Stay tune -------- Quitting for now")
        return

    if args.datefrom:
        date_from = args.datefrom.replace('/', '.')
    if args.dateto:
        date_to = args.dateto.replace('/', '.')
    
    show_ads = False
    if args.ads:
        show_ads = True
    
    #process_cds=args.cds
    process_cds=True
    #output=False
    #compress=False
    verbose_level = args.verbose
    quiet=False
    if verbose_level == 0:
        quiet=True
    #print("Verbose level : " + str(verbose_level))
    if process_cds:
        print("Processing CDS")
        output=True
    #if args.compress:
    #    compress = args.compress
    #     output = True # in case
    # if args.output:
    #     output = True

    if verbose_level > 1:
        if date_from:
            print("Date from : " + str(date_from))
        if date_to:
            print("Date to : " + str(date_to))


    try:
        
        print_quiet(quiet,"Getting for : " + instrument + "_" + timeframe)
        instruments = instrument.split(',')
        timeframes = timeframe.split(',')


        for instrument in instruments:
            for timeframe in timeframes:
                createCDS_for_main(instrument, timeframe, quiet=quiet, verbose_level=verbose_level,tlid_range=tlid_range,show_ads=show_ads,cc=cc)
                # else:
                #     p = pds.getPH(instrument, timeframe, quotes_count, date_from, date_to, False, quiet)
                #     if verbose_level > 0:
                #         print(p)
         
    except Exception as e:
        jgtcommon.print_exception(e)

    #try:
    #    jgtpy.off()
    #except Exception as e:
    #    jgtfxcommon.print_exception(e)

# if __name__ == "__main__":
#     main()

# print("")
# #input("Done! Press enter key to exit\n")

def createCDS_for_main(instrument, timeframe, quiet, verbose_level=0,tlid_range=None,show_ads=False,cc: JGTChartConfig = None):
    # implementation goes here
    col2remove=constants.columns_to_remove
    config = jgtcommon.readconfig()
    if 'columns_to_remove' in config:  # read it from config otherwise
        col2remove = config['columns_to_remove']
    quietting=True
    if verbose_level> 1:
        quietting=False
    try: 
        cdspath,cdf=cds.createFromPDSFileToCDSFile(instrument,timeframe,col2remove) #@STCIssue: This is not supporting -c NB_BARS_TO_PROCESS, should it ? 
        if show_ads:#(data,instrument,timeframe,nb_bar_on_chart = 375,show=True,plot_ao_peaks=False)
            ads.plot_from_cds_df(cdf,instrument,timeframe,show=True,plot_ao_peaks=True,cc=cc) 
        print_quiet(quiet,cdspath)
        print_quiet(quiet,cdf)
    except Exception as e:
        print("Failed to create CDS for : " + instrument + "_" + timeframe)
        print("Exception: " + str(e))
    



def print_quiet(quiet,content):
    if not quiet:
        print(content)



if __name__ == "__main__":
    main()



# %%

import sys
import os

sys.path.insert(0, os.path.abspath(os.path.dirname(__file__)))

#import jgtfxcon.JGTPDS as pds
import JGTIDS as ids
import JGTPDSP as pds
from jgtutils.jgtos import get_data_path
from JGTChartConfig import JGTChartConfig
#from . import jgtconstants
#.columns_to_remove as columns_to_remove

import pandas as pd


from jgtutils import jgtconstants as c


# %%
def createFromPDSFileToCDSFile(instrument, timeframe, columns_to_remove=None, quiet=True,tlid_range=None):
  """
  Create a CDS file from a PDS file.

  Parameters:
  - instrument (str): The instrument name.
  - timeframe (str): The timeframe of the data.
  - columns_to_remove (list, optional): List of column names to remove from the CDS file. Default is None.
  - quiet (bool, optional): If True, suppresses the output. Default is True.
  - tlid_range (str, optional): The TLID range to retrieve. Default is None.

  Returns:
  - fpath (str): The file path of the created CDS file.
  - c (DataFrame): The DataFrame containing the data.

  """
  cdf = createFromPDSFile(instrument, timeframe, quiet,tlid_range=tlid_range)

  # Remove the specified columns
  if columns_to_remove is not None:
    cdf = cdf.drop(columns=columns_to_remove, errors='ignore')

  # # Reset the index
  # try:
  #   c.reset_index(inplace=True)
  # except:
  #   pass

  # Define the file path based on the environment variable or local path
  data_path_cds = get_data_path("cds")
  fpath = pds.mk_fullpath(instrument, timeframe, 'csv', data_path_cds)
  #print(fpath)
  cdf.to_csv(fpath)

  return fpath, cdf

def readCDSFile(instrument, timeframe, columns_to_remove=None, quiet=True):
  """
  Read a CDS file and return a pandas DataFrame.

  Parameters:
  instrument (str): The instrument name.
  timeframe (str): The timeframe of the data.
  columns_to_remove (list, optional): List of column names to remove from the DataFrame. Default is None.
  quiet (bool, optional): If True, suppresses the output messages. Default is True.

  Returns:
  pandas.DataFrame: The DataFrame containing the CDS data.
  """
  # Define the file path based on the environment variable or local path
  data_path_cds = get_data_path("cds")
  fpath = pds.mk_fullpath(instrument, timeframe, 'csv', data_path_cds)
  cdf = pd.read_csv(fpath)

  # Set 'Date' as the index
  cdf.set_index('Date', inplace=True)
  # Remove the specified columns
  if columns_to_remove is not None:
    cdf = cdf.drop(columns=columns_to_remove, errors='ignore')
  return cdf

def createFromPDSFile(instrument,timeframe,quiet=True,tlid_range=None,cc: JGTChartConfig = None):
  """Create CDS (Chaos Data Service) with Fresh Data on the filestore

  Args:
      instrument (str): symbol
      timeframe (str): TF
      quiet (bool,optional): Output quiet
      tlid_range (str,optional): TLID range
      cc (JGTChartConfig, optional): The JGTChartConfig object to use for the processing. Defaults to None.
      cc (JGTChartConfig, optional): The JGTChartConfig object to use for the processing. Defaults to None.

  Returns:
      pandas.DataFrame: CDS DataFrame
  """
  try:
    df=pds.getPH_from_filestore(instrument,timeframe,quiet=quiet,tlid_range=tlid_range)
    if not quiet:
      print(df)
  
    dfi=createFromDF(df,quiet=quiet,cc=cc)
    return dfi
  except:
    return None

def createFromDF(df, quiet=True,cc: JGTChartConfig = None):
  """
  Creates a new DataFrame with indicators, signals, and cleansed columns added based on the input DataFrame.

  Args:
    df (pandas.DataFrame): The input DataFrame to add indicators, signals, and cleansed columns to.
    quiet (bool, optional): Whether to suppress console output during processing. Defaults to True.
    cc (JGTChartConfig, optional): The JGTChartConfig object to use for the processing. Defaults to None.

  Returns:
    pandas.DataFrame: The new DataFrame with indicators, signals, and cleansed columns added.
  """
  if cc is None:
    cc = JGTChartConfig()
    
  if df.index.name == 'Date':
      df.reset_index(inplace=True)
  dfi=ids.tocds(df,quiet=quiet,cc=cc) 
  return dfi


def create(instrument,timeframe,nb2retrieve=335,stayConnected=False,quiet=True,cc: JGTChartConfig = None):
  """Create CDS (Chaos Data Service) with Fresh Data

  Args:
      instrument (str): symbol
      timeframe (str): TF
      nb2retrieve (int, optional): nb bar to retrieve. Defaults to 335.
      stayConnected (bool, optional): Leave Forexconnect connected. Defaults to False.
      quiet (bool,optional): Output quiet
      cc (JGTChartConfig, optional): The JGTChartConfig object to use for the processing. Defaults to None.

  Returns:
      pandas.DataFrame: CDS DataFrame
  """
  
  df=pds.getPH(instrument,timeframe,nb2retrieve,with_index=False,quiet=quiet,cc=cc)
  dfi=createFromDF(df,quiet=quiet,cc=cc)
  return dfi
  

#createByRange
def createByRange(instrument,timeframe,start,end,stayConnected=False,quiet=True):
  """Create CDS with Fresh Data from a range

  Args:
      instrument (str): symbol
      timeframe (str): TF
      start (date): start date
      end (date): end date
      stayConnected (bool, optional): Leave FXCMPY connected. Defaults to False.
      quiet (bool,optional): Output quiet

  Returns:
      pandas.DataFrame: CDS DataFrame
  """
  pds.stayConnected=stayConnected
  df=pds.getPHByRange(instrument,timeframe,start,end,with_index=False,quiet=quiet)
  dfi=createFromDF(df,quiet=quiet)
  return dfi





columns_to_remove = ['aofvalue', 'aofhighao', 'aoflowao', 'aofhigh', 'aoflow', 'aocolor', 'accolor', 'fdbbhigh', 'fdbblow', 'fdbshigh', 'fdbslow']
def create_and_clean_data_from_file_df(instrument, timeframe):
    # Create DataFrame from PDS file
    cdf = createFromPDSFile(instrument, timeframe)

    # Remove specified columns if provided
    if columns_to_remove:
        cdf = cdf.drop(columns=columns_to_remove, errors='ignore')

    # Set 'Date' as the index
    cdf.set_index(c.DATE, inplace=True)

    return cdf


  
def _save_cds_data_to_file(df, instrument, timeframe):
    # Define the file path based on the environment variable or local path
    data_path = get_data_path()
    fpath = pds.mk_fullpath(instrument, timeframe, 'csv', data_path)

    # Save DataFrame to CSV
    df.to_csv(fpath)
    return fpath

def createFromFile_and_clean_and_save_data(instrument, timeframe):
    # Create DataFrame from PDS file
    cdf =create_and_clean_data_from_file_df(instrument, timeframe)
    _save_cds_data_to_file(cdf, instrument, timeframe)


    return cdf









def getSubscribed():
  return pds.getSubscribed()

def getActiveSymbols():
  AppSuiteConfigRootPath = os.getenv('AppSuiteConfigRootPath')
  fn='activesymbol.txt'
  fpath=os.path.join(AppSuiteConfigRootPath,fn)
  with open(fpath) as f:
    first_line = f.readline()
    print(first_line)

# %%
def getLast(_df):
  return _df.iloc[-1]

def getPresentBar(_df):
  r= _df#['High','Low',indicator_AO_awesomeOscillator_column_name,signalCode_fractalDivergentBar_column_name,indicator_AC_accelerationDeceleration_column_name]
  return r.iloc[-1:]

def getPresentBarAsList(_df):
  _paf =_df.iloc[-1:]
  _pa = _paf.to_dict(orient='list')
  _dtctx=str(_paf.index.values[0])
  _pa['Date'] = _dtctx
  return _pa


def getLastCompletedBarAsList(_df):
  _paf =_df.iloc[-2:-1]
  _pa = _paf.to_dict(orient='list')
  _dtctx=str(_paf.index.values[0])
  _pa['Date'] = _dtctx
  return _pa





def checkFDB(_instrument,_timeframe):
  _df=create(_instrument)
  pa = getPresentBarAsList(_df)
  isfdb = pa[c.FDB][0]  != 0.0
  fdb = pa[c.FDB]
  dtctx = pa[c.DATE]
  if isfdb:
    print(_instrument + "_" + _timeframe + " : We Have a Signal : " + dtctx)
    return True
  else:
    print(_instrument + "_" + _timeframe +  " : No signal now : " + dtctx)
    return False



def print_quiet(quiet,content):
    if not quiet:
        print(content)







import datetime as dt
import pandas as pd
import os
import json
import sys

sys.path.insert(0, os.path.abspath(os.path.dirname(__file__)))

import JGTPDHelper as jpd

from JGTConfig import local_fn_compression,get_pov_local_data_filename
from JGTPDHelper import *
from datetime import datetime
from jgtutils import jgtcommon,iprops,jgtos
from jgtutils.jgtos import create_filestore_path, mk_fn, mk_fn_range, mk_fullpath

from JGTChartConfig import JGTChartConfig

renameColumns=True
addOhlc=True


cleanseOriginalColumns=True
useLocal=True


def getPH(instrument, timeframe, quote_count=-1, start=None, end=None, with_index=True, quiet=True,convert_date_index_to_dt=True,cc: JGTChartConfig=None,get_them_all=False):
  #@STCissue quote_count is ignored or irrelevant in start/end
  #@a Adequate start and end from the stored file
  if cc is None:
    cc = JGTChartConfig()
  if quote_count == -1:
    quote_count = cc.nb_bar_to_retrieve
  
  df = getPH_from_filestore(instrument, timeframe, quiet, False, with_index,convert_date_index_to_dt)
  if not quiet:
    print(df.columns)
    print(df.index)
    print("------------------------------------")
  
  if start is not None:
    #@STCIssue Not supported supplying the 'end' with a count
    if end is None:  # end is not provided
      end = datetime.now()
    
    if not quiet:
      print("start: " + str(start))
      print("end: " + str(end))
    df = select_start_end(df, start, end)
  ldf = len(df)
  if ldf > quote_count and not get_them_all:
    df = df.iloc[-quote_count:]
  return df

def select_start_end(df, start, end=None):
  if end is None:  # end is not provided
    end = datetime.now()
  
  if 'Date' in df.columns:
    mask = (df['Date'] >= start) & (df['Date'] <= end)
  else:
    mask = (df.index >= start) & (df.index <= end)
  
  selected_df = df.loc[mask]
  return selected_df


def str_to_datetime(date_str):
    formats = ['%m.%d.%Y %H:%M:%S', '%Y-%m-%d %H:%M:%S', '%Y/%m/%d %H:%M:%S', '%Y/%m/%d', '%Y-%m-%d']
    
    for fmt in formats:
        try:
            return datetime.strptime(date_str, fmt)
        except ValueError:
            continue
    return None


def getPH_from_filestore(instrument,timeframe,quiet=True, compressed=False,with_index=True,convert_date_index_to_dt=True,
                         tlid_range=None,
                         output_path=None,
                         nsdir="pds"):
  """
  Retrieves OHLC data for a given instrument and timeframe from the filestore.

  Args:
    instrument (str): The instrument symbol.
    timeframe (str): The timeframe of the OHLC data.
    quiet (bool, optional): Whether to suppress print statements. Defaults to True.
    compressed (bool, optional): Whether the data is compressed. Defaults to False.
    with_index (bool, optional): Whether to include the index in the returned DataFrame. Defaults to True.
    convert_date_index_to_dt  (bool, optional): convert index Date to dt
    tlid_range (str, optional): Select a range on disk or return None if unavailable
    output_path (str, optional): The path to the output directory. Defaults to None.
    nsdir (str, optional): The name of the directory to use for the filestore. Defaults to "pds".
    

  Returns:
    pandas.DataFrame: The OHLC data for the given instrument and timeframe.
  """  
  srcpath = create_filestore_path(instrument, timeframe,quiet, compressed,tlid_range=tlid_range,output_path=output_path,nsdir=nsdir)
  
  print_quiet(quiet,srcpath)
  
  df = None
  try:
    df = read_ohlc_df_from_file(srcpath,quiet,compressed,with_index,convert_date_index_to_dt)
  except:
    pass 
  return df




def read_ohlc_df_from_file(srcpath, quiet=True, compressed=False,with_index=True,convert_date_index_to_dt=True):
  """
  Reads an OHLC (Open-High-Low-Close) +Date as DataFrame index from a CSV file.

  Args:
    srcpath (str): The path to the CSV file.
    quiet (bool, optional): Whether to print progress messages. Defaults to True.
    compressed (bool, optional): Whether the CSV file is compressed. Defaults to False.

  Returns:
    pandas.DataFrame: The OHLC DataFrame.
  """
  df = None
  try:
    if compressed:
      print_quiet(quiet, "Reading compressed: " + srcpath + " ")
      df = pd.read_csv(srcpath, compression=local_fn_compression)
    else:
      print_quiet(quiet, "Reading uncompressed csv file: " + srcpath)
      df = pd.read_csv(srcpath)
  except Exception as e:
    print(f"An error occurred while reading the file: {e}")
    df = None
  if with_index:
    if 'Date' in df.columns:
      df.set_index('Date', inplace=True)
      
      if convert_date_index_to_dt:
        df.index = pd.to_datetime(df.index)
    else:
      raise ValueError("Column 'Date' is not present in the DataFrame")
  return df
 

# def create_filestore_path(instrument, timeframe,quiet=True, compressed=False,tlid_range=None,output_path=None,nsdir="pds"):
#   return jgtos.create_filestore_path(instrument,timeframe,quiet=quiet,compressed=compressed,tlid_range=tlid_range,output_path=output_path,nsdir=nsdir)
  
  
# def mk_fn(instrument,timeframe,ext="csv"):
#   return jgtos.mk_fn(instrument,timeframe,ext)


# def mk_fn_range(instrument, timeframe, start: datetime, end: datetime,ext="csv"):
#   return jgtos.mk_fn_range(instrument,timeframe,start,end,ext)


# def mk_fullpath(instrument,timeframe,ext,path,tlid_range=None):
#   return jgtos.mk_fullpath(instrument,timeframe,ext,path,tlid_range)


def get_data_path():
    return jgtos.get_data_path('pds')
  
  

def get_instrument_properties(instrument, quiet=False,from_file=True):
  if not from_file:
    print("NOT SUPORTED in PDSP")
  else:
    
    # # Define the path to the directory
    instrument_properties = {}
    try:
      instrument_properties = iprops.get_iprop(instrument)
      instrument_properties["pipsize"] = instrument_properties["pips"]
    except:
      home_dir = os.path.expanduser("~")
      dir_path = os.path.join(home_dir, '.jgt', 'iprops')
      instrument_filename = instrument.replace('/', '-')
      #     # Read the instrument properties from the file
      iprop_dir_path = os.path.join(dir_path, f'{instrument_filename}.json')
      with open(iprop_dir_path, 'r') as f:
        instrument_properties = json.load(f)
    return instrument_properties


def print_quiet(quiet,content):
    if not quiet:
        print(content)
        



# Copyright 2023 Jean Guillaume Isabelle
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
.. moduleauthor:: Jean Guillaume Isabelle <jgi@jgwill.com>
"""


import os
import platform
import sys

import sys
import os

sys.path.insert(0, os.path.abspath(os.path.dirname(__file__)))


import jgtflags


import warnings

with warnings.catch_warnings():
    warnings.filterwarnings(
        "ignore", category=RuntimeWarning, module="importlib._bootstrap"
    )
    # your code here


class NotCompatibleException(Exception):
    pass


# from jgtpy.common_samples import common_samples
from jgtetl import svc_offset_dt_by_tf as etl_offset_dt_by_tf, offsetdt as etl_offsetdt
from JGTCore import (
    __version__,
)  # ,json2dict,jsonfile2prop,json2prop,jsonfile2dict,d2p,fixdtindf,offsetdt

# from .JGTConfig import getenv,setreal,setdemo,env


from jgtutils import jgtwslhelper as wsl

import JGTIDS as ids, JGTADS as ads, JGTPDSP as pds, JGTCDS as cds

from JGTIDS import tocds as tocds

# mk_fn,mk_fullpath,getSubscribed,getPH,getPHByRange,tryConnect
from JGTCDS import (
    create as createCDS,
    createByRange,
    createFromDF,
    getLast,
    getPresentBar,
    getPresentBarAsList,
    getLastCompletedBarAsList,
    createFromDF,
    createFromFile_and_clean_and_save_data as fromfile,
    createFromFile_and_clean_and_save_data as ff,
    createFromDF as fp,
)
from JGTADS import plot as plot
import jgtpy.JGTMKSG as mksg


def help():
    print(
        ".h(i,t,400)\t\tGet Prices (PDS): \n\t\t\tjgtpy.h(instrument,timeframe,quote_count=335,start=None,end=None,quiet=True)"
    )
    print("\t\t\t\t\treturn DataFrame of PDS Type")
    print("\t\t\t\t\t(Will connect and stay connected)")
    print(".fp(df)\t\t Create CDS from PDS DF \n\t\t\tjgtpy.fp(df [pd.DataFrame])")
    print("\t\t\t\t\treturn DataFrame of CDS Type")
    print(".off()\t\t Disconnect (Bugged)\n\t\t\tjgtpy.off()")



class JGTChartConfig:
    def __init__(self):
        
        self.show_grid = False
        
        # subtitle & title
        self.subtitle_x_pos = 0.07
        self.subtitle_y_pos = 0.9
        self.subtitle_ha = "left"
        self.subtitle_fontsize = 10
        
        self.title_x_pos = 0.055
        self.title_y_pos = 0.96
        self.title_ha = "left"
        self.title_fontsize = 14
        
        
        self.nb_bar_on_chart = 300
        self.min_bar_on_chart = 299  # Somehow it not that helpful because some instruments has less than 100 bars on their montly chart and close to 300 an their weekly chart
        self.b_alligator_jaws_period = 89
        self.cds_required_amount_of_bar_for_calc = self.b_alligator_jaws_period
        self.nb_bar_to_retrieve = (
            self.nb_bar_on_chart + self.cds_required_amount_of_bar_for_calc
        )

        self.jaw_line_width = 1
        self.teeth_line_width = 1
        self.lips_line_width = 1
        self.fig_ratio_x = 24
        self.fig_ratio_y = 14
        self.fdb_marker_size = 7
        self.fractal_marker_size = 6
        self.ac_signals_marker_size = 32
        self.saucer_marker_size = 16
        self.fractal_degreehigher_marker_size = 20
        self.price_peak_marker_size = 36
        self.ao_peaks_marker_size = 42
        self.fdb_signal_marker = "o"
        self.fractal_up_marker = "^"
        self.fractal_up_marker_higher = "^"
        self.fractal_dn_marker_higher = "v"
        self.fractal_dn_marker = "v"
        self.ac_signal_marker = "o"
        self.acb_plot_type = "scatter"

        self.ao_peak_above_marker_higher = "^"
        self.ao_peak_bellow__marker_higher = "v"
        self.saucer_marker = "|"
        self.zcol_marker = "|"
        self.aop_bellow_color = "r"
        self.aop_above_color = "g"

        self.ao_upbar_color = "g"
        self.ao_dnbar_color = "r"
        self.ac_up_color = "darkgreen"
        self.ac_dn_color = "darkred"
        self.fdb_signal_buy_color = "g"
        self.fdb_signal_sell_color = "r"
        self.jaw_color = "blue"
        self.teeth_color = "red"
        self.lips_color = "green"
        self.fractal_up_color = "blue"
        self.fractal_dn_color = "blue"
        self.fractal_dn_color_higher = "blue"
        self.fractal_up_color_higher = "blue"
        self.ac_signal_buy_color = "lightgreen"
        self.ac_signal_sell_color = "yellow"
        self.saucer_buy_color = "g"
        self.saucer_sell_color = "r"
        self.price_peak_bellow_marker = "o"
        self.price_peak_above_marker = "o"
        self.price_peak_above_color = "g"
        self.price_peak_bellow_color = "r"

        self.ao_peak_offset_value = 0

        self.plot_style = "yahoo"
        self.ao_plot_type = "bar"
        self.ac_plot_type = "bar"

        self.main_plot_panel_id = 0
        self.ao_plot_panel_id = 1
        self.ac_plot_panel_id = 2


# Create an instance with default values
default_config = JGTChartConfig()



from . import JGTCDS as cds

instrument="EUR/USD"
timeframe = "D1"
c=cds.create(instrument,timeframe,nb2retrieve=3000,stayConnected=False,quiet=True)
print(c)
c.to_csv("../test/output-cds-231109a.csv")






#@STCGoal Aim to become the container for lighting JGTADS

import sys
import os
sys.path.insert(0, os.path.abspath(os.path.dirname(__file__)))
import JGTPDSP as pds
import JGTIDS as ids
import JGTCDS as cds
from jgtutils import jgtwslhelper as wsl
import pandas as pd

from JGTChartConfig import JGTChartConfig

import logging
_loglevel= logging.WARNING

# Create a logger object
l = logging.getLogger()
l.setLevel(_loglevel)

# Create a console handler and set its level
console_handler = logging.StreamHandler()
console_handler.setLevel(_loglevel)

# Create a formatter and add it to the console handler
formatter = logging.Formatter('%(asctime)s - %(levelname)s - %(message)s')
console_handler.setFormatter(formatter)

# Add the console handler to the logger
l.addHandler(console_handler)



def read_csv(csv_fn):
    df=pd.read_csv(csv_fn)
    # try:
    #     df.set_index('Date', inplace=True)
    # except:
    #     pass
    try:
        df.drop(columns=['Unnamed: 0'],inplace=True)
    except:
        pass
    return df

#IN_CHART_BARS=300

def prepare_cds_for_ads_data(instrument:str, timeframe:str,tlid_range:str=None,cc:JGTChartConfig=None,crop_last_dt:str=None):
    """
    Prepare CDS (Credit Default Swap) data for ADS (Automated Trading System).

    Args:
        instrument (str): The instrument symbol.
        timeframe (str): The timeframe of the data.
        tlid_range (str, optional): The range of TLID to select. Defaults to None.
        cc (JGTChartConfig, optional): The chart configuration. Defaults to None.
        crop_last_dt (str, optional): The last date to crop the data. Defaults to None.
        
    Returns:
        pandas.DataFrame: The prepared CDS data with Selected number of bars

    """
    if cc is None:
        cc = JGTChartConfig()
        
    #@STCIssue Deprecating this value for later 
    
    #print("AH:DEBUG::Tlid_range:",tlid_range)
    if tlid_range is not None:
        raise NotImplementedError("tlid_range is not implemented yet. We will use crop_last_dt instead.")
    #@STCGoal local retrieve data from cache if available or from WSL if not  (jgtfxcli)
        
    cache_data=False
    cache_dir = "cache"
    if cache_data:
        os.makedirs(cache_dir, exist_ok=True)

    fn =  instrument.replace("/", "-") + "_" + timeframe + ".csv"
    fnpath = os.path.join(cache_dir,fn)
    l.info("fnpath:"+ fnpath)

    # @STCIssue: ,get_them_all=False
    
    try:
        df = pds.getPH(instrument,timeframe,cc=cc,get_them_all=True)
        #print(str(len(df))+" rows")
        #print("------------------")
        #drop the latest rows until we have the crop_last_dt as last date if not None
        # crop_last_dt="2022-10-13 13:45:00"
        if crop_last_dt is not None:
            df = df[df.index <= crop_last_dt]
            #print("After dropping the latest rows until we have the crop_last_dt as last date if not None")
            #print(str(len(df))+" rows")
            #print("------------------")
        #print(str(len(df))+" rows")
        #print("------------------")
        #print("pds df:",str(len(df))+" rows")
    except:
        l.warning("Could not get DF, trying to run thru WSL the update")
        wsl.jgtfxcli(instrument, timeframe, cc.nb_bar_to_retrieve)
        df = pds.getPH(instrument,timeframe,cc.nb_bar_to_retrieve) #@STCIssue Limitation of full range to be given yo jgtfxcli
    # Select the last 400 bars of the data
    try:#@q Is the selected correspond to desirrd bars ?
        #Make sure we have enough bars to select
        nb_to_select = cc.nb_bar_to_retrieve
        if nb_to_select < cc.min_bar_on_chart:
            nb_to_select = cc.min_bar_on_chart
            selected = df.copy()
        else:
            selected = df.iloc[-nb_to_select:].copy()
        
        #selected.to_csv("output_ads_prep_data.csv")
    except:
        l.warning("Could not get DF, trying to run thru WSL the update")
        wsl.jgtfxcli(instrument, timeframe, cc.nb_bar_to_retrieve)
        try:
            df = pds.getPH(instrument,timeframe,cc.nb_bar_to_retrieve)
            selected = df.copy()
        except:
            l.warning("Twice :(Could not select the desired amount of bars, trying anyway with what we have")
            pass
        l.warning("Could not select the desired amount of bars, trying anyway with what we have")
        pass
    #print(selected)
    len_selected = len(selected)
    #print("Len_selected:",len_selected)
    #print(selected.tail(1))
    #print("---------------")
    data = cds.createFromDF(selected)
    if cache_data:
        data.to_csv(fnpath)
        
    nb_bars = len(data)
    #print("AH:Debug: nb_bar_on_chart:",nb_bar_on_chart)
    #print("AH:Debug:nb_bars b4 prep ends well:",nb_bars)
    if nb_bars> cc.nb_bar_on_chart:
        r = data.iloc[-cc.nb_bar_on_chart:].copy()
    else:
        r= data
    #len_r = len(r)
    #print("AH:Debug:nb_bars after prep ends well:",len_r)
    return r


def get(instrument:str, timeframe:str,tlid_range:str=None,cc:JGTChartConfig=None,crop_last_dt:str=None):
#(instrument,timeframe,nb_bar_on_chart=500,crop_last_dt:str=None):
  data = prepare_cds_for_ads_data(instrument, timeframe,tlid_range,cc,crop_last_dt)
  return data
#p=pds.getPH(instrument,timeframe)



from flask import Flask, jsonify, request
#from jgtpy import sc,up,h,stay

from subprocess import check_output
import shlex


app = Flask(__name__)
#stayConnectedSetter(True)


@app.route('/run_jgtcli', methods=['POST'])
def run_jgtcli():
    data = request.json
    instrument = data['instrument']
    timeframe = data['timeframe']

    # Optional parameters with default values
    datefrom = data.get('datefrom', None)
    dateto = data.get('dateto', None)
    #output = '-o' if data.get('output', False) else ''
    cds = '-cds' if data.get('cds', False) else ''
    verbose = '-v %s' % data.get('verbose', 0)

    # Construct the command
    cmd = f'jgtcli -i {instrument} -t {timeframe} -o {cds} {verbose}'


    if datefrom:
        cmd += f' -s "{datefrom}"'
    if dateto:
        cmd += f' -e "{dateto}"'

    # Execute the command
    print("==================CLI===================")
    print(cmd)
    print("========================================")

    try:
        result = check_output(shlex.split(cmd)).decode('utf-8')
        return jsonify({'result': result})
    except Exception as e:
        return jsonify({'error': str(e)})



# More routes for other functions...

if __name__ == '__main__':
    app.run(debug=True)





import unittest
import datetime
from jgtos import tlid_range_to_jgtfxcon_start_end_str
from jgtos import tlid_range_to_start_end_datetime

class TestTlidRangeToJgtfxconStartEndStr(unittest.TestCase):
    def test_tlid_range_to_jgtfxcon_start_end_str(self):
        # Arrange
        tlid_range = "211124_220113"
        expected_start = '11.24.2021 00:00:00'
        expected_end = '01.13.2022 00:00:00'

        # Act
        result_start, result_end = tlid_range_to_jgtfxcon_start_end_str(tlid_range)

        # Assert
        self.assertEqual(str(result_start), expected_start)
        self.assertEqual(str(result_end), expected_end)
        
        
        tlid_range = "2111242222_2201132323"
        expected_start = '11.24.2021 22:22:00'
        expected_end = '01.13.2022 23:23:00'

        # Act
        result_start, result_end = tlid_range_to_jgtfxcon_start_end_str(tlid_range)

        # Assert
        self.assertEqual(result_start, expected_start)
        self.assertEqual(result_end, expected_end)
        
        
        
        tlid_range = "9611242222_0101132323"
        expected_start = '11.24.1996 22:22:00'
        expected_end = '01.13.2001 23:23:00'

        # Act
        result_start, result_end = tlid_range_to_jgtfxcon_start_end_str(tlid_range)

        # Assert
        self.assertEqual(str(result_start), expected_start)
        self.assertEqual(str(result_end), expected_end)





class TestTlidRangeToStartEndDatetime(unittest.TestCase):
    def test_tlid_range_to_start_end_datetime_with_time(self):
        # Arrange
        tlid_range = "2111242200_2201132359"
        expected_start = datetime.datetime(2021, 11, 24, 22, 0)
        expected_end = datetime.datetime(2022, 1, 13, 23, 59)

        # Act
        result_start, result_end = tlid_range_to_start_end_datetime(tlid_range)

        # Assert
        self.assertEqual(result_start, expected_start)
        self.assertEqual(result_end, expected_end)

    def test_tlid_range_to_start_end_datetime_with_date(self):
        # Arrange
        tlid_range = "2111240010_2201132359"
        expected_start = datetime.datetime(2021, 11, 24,0,10)
        expected_end = datetime.datetime(2022, 1, 13, 23, 59)

        # Act
        result_start, result_end = tlid_range_to_start_end_datetime(tlid_range)

        # Assert
        self.assertEqual(result_start, expected_start)
        self.assertEqual(result_end, expected_end)

    def test_tlid_range_to_start_end_datetime_invalid_format(self):
        # Arrange
        tlid_range = "992345_2201131122"
        expected_result = None

        # Act
        result = tlid_range_to_start_end_datetime(tlid_range)

        # Assert
        self.assertEqual(result, expected_result)


if __name__ == "__main__":
    unittest.main()





# class JGTDotenv(dict):
#     def __init__(self, file_path):
#         self.file_path = file_path
#         super(JGTDotenv, self).__init__(**self.__create_dict())

#     def __create_dict(self):
#         with open(self.file_path, 'r') as jgtdotenv:
#             variables = {}
#             for line in jgtdotenv.readlines():
#                 variables.update(self.__parse_line(line))
#             return variables

#     def __parse_line(self, line):
#         if line.lstrip().startswith('#'):
#             # discard and return nothing
#             return {}
#         if line.lstrip():
#             # find the second occurence of a quote mark:
#             quote_delimit = max(line.find('\'', line.find('\'') + 1),
#                                 line.find('"', line.rfind('"')) + 1)
#             # find first comment mark after second quote mark
#             comment_delimit = line.find('#', quote_delimit)
#             line = line[:comment_delimit]
#             key, value = map(lambda x: x.strip().strip('\'').strip('"'),
#                              line.split('=', 1))
#             return {key: value}
#         else:
#             return {}

#     def __persist(self):
#         with open(self.file_path, 'w') as jgtdotenv:
#             for key, value in self.items():
#                 jgtdotenv.write("%s=%s\n" % (key, value))

#     def __setitem__(self, key, value):
#         super(JGTDotenv, self).__setitem__(key, value)
#         self.__persist()

#     def __delitem__(self, key):
#         super(JGTDotenv, self).__delitem__(key)
#         self.__persist()


# def set_variable(file_path, key, value):
#     jgtdotenv = JGTDotenv(file_path)
#     jgtdotenv[key] = value


# def get_variable(file_path, key):
#     jgtdotenv = JGTDotenv(file_path)
#     return jgtdotenv[key]


# def get_variables(file_path):
#     return JGTDotenv(file_path)




# # # encoding=UTF-8

# # from __future__ import with_statement


# # __versionjgtdotenv__ = '0.0.5'
# # import argparse
# # #from jgtdotenv import get_variable, set_variable, get_variables, __version__


# # parser = argparse.ArgumentParser()

# # parser.add_argument("key", nargs='?')
# # parser.add_argument("value", nargs='?')

# # parser.add_argument('--file', default='.env')

# # parser.add_argument('--version', action='version', version=__version__)

# # parser.add_argument('--shell', action='store_true', default=False)

# # args = parser.parse_args()


# # if args.shell:
# #     PRINT_FORMAT = '%s=%s'
# # else:
# #     PRINT_FORMAT = '%s: %s'

# # if args.key is None:
# #     for key, value in get_variables(args.file).items():
# #         print(PRINT_FORMAT % (key, value))
# # elif args.value is None:
# #     print(PRINT_FORMAT % (args.key, get_variable(args.file, args.key)))
# # else:
# #     set_variable(args.file, args.key, args.value)
# #     print(PRINT_FORMAT % (args.key, args.value))

